import { Component, OnInit } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";

import { MainService } from "../../services/main.service";
import { ToolService } from "../../services/tool.service";
import { Subject } from "rxjs";
import { debounceTime } from "rxjs/operators";
import { NzMessageService } from 'ng-zorro-antd/message';
import { format } from 'date-fns';
import { FormGroup, Validators, FormBuilder } from '@angular/forms';
@Component({
  selector: "app-incident-management",
  templateUrl: "./incident-management.component.html",
  styleUrls: ["./incident-management.component.less"],
})
export class IncidentManagementComponent implements OnInit {
  constructor(
    public route: ActivatedRoute,
    private router: Router,
    private mainService: MainService,
    private tool: ToolService,
    private message: NzMessageService,
    private fb: FormBuilder,
  ) {}
  loginUser: any = localStorage.getItem("user")
    ? JSON.parse(localStorage.getItem("user")).user
    : null; //登录人信息
  listOfData: any[] = []; //表格数据
  coopId: string; //表格中执行操作的id
  hospital: string; //选中院区
  alldepart: any = []; //当前院区所属科室
  worker: number; //选择执行配送人员
  allWorker: any = []; //当前院区执行配送人员列表
  gdState: number; //选择工单状态
  gdStates: any; //工单状态列表
  pageIndex: number = 1; //页码
  listLength: number = 10; //总条数
  pageSize: number = 10; //每页条数

  promptContent: string; //操作提示框提示信息
  ifSuccess: boolean; //操作成功/失败
  promptInfo: string; //操作结果提示信息
  promptModalShow: boolean; //操作提示框是否展示

  btnLoading: boolean = false; //提交按钮loading状态

  tabs:any[] = [
    // {key: 'all', value: '全部故障', num: 0},
    {key: 'todo', value: '待我接单', num: 0},
    {key: 'doing', value: '待我处理', num: 0},
    // {key: 'reassign', value: '重新指派', num: 0},
    // {key: 'callback', value: '待我回访', num: 0},
    {key: 'resolve', value: '由我解决', num: 0},
    {key: 'owns', value: '与我关联', num: 0},
    {key: 'storage', value: '暂存', num: 0},
    // {key: 'badEvaluate', value: '异常评价', num: 0},
  ]

  searchDTO: any = {};

  searchTimerSubject = new Subject();

  // 初始化增删改按钮
  coopBtns: any = {};

  // 选择tab
  queryTask:string = 'todo';//默认待我接单
  changeTab(key){
    this.queryTask = key;
  }

  ngOnInit() {
    this.searchTimerSubject.pipe(debounceTime(500)).subscribe((v) => {
      let fun = v[0];
      fun.call(this, v[1]);
    });
    this.coopBtns = this.tool.initCoopBtns(this.route);
    this.initTabs();
    this.getAllHos();
    this.getGdStates();
  }

  // 初始化tab
  initTabs(){
    if (this.coopBtns.all) {
        this.tabs.splice(0, 0 , {key: 'all', value: '全部故障', num: 0});
    }
    if (this.coopBtns.callback) {
        let index = this.tabs.findIndex(v => v.key == 'resolve');
        this.tabs.splice(index, 0 , {key: 'callback', value: '待我回访', num: 0});
    }
    if (this.coopBtns.reassign) {
        let index = this.tabs.findIndex(v => v.key == 'doing');
        this.tabs.splice(index + 1, 0 , {key: 'reassign', value: '重新指派', num: 0});
    }
    if (this.coopBtns.badEvaluate) {
        this.tabs.push({key: 'badEvaluate', value: '异常评价', num: 0});
    }
  }

  // 搜索
  search() {
    this.pageIndex = 1;
    this.getList();
  }
  // 重置
  reset() {
    this.pageIndex = 1;
    if (!(this.coopBtns.currentDept && !this.coopBtns.allOrders)) {
      this.searchDTO.department = null;
    }
    if (!(this.coopBtns.currentUser && !this.coopBtns.allOrders)) {
      this.worker = null;
    }
    this.gdState = null;
    this.searchDTO = {};
    this.changeDate();
    this.getList();
  }

  // 获取院区
  getAllHos() {
    this.hospital = this.tool.getCurrentHospital().id + "";
    this.changeHos();
  }
  // 修改院区获取对应科室,工单类型，执行配送人员
  changeHos() {
    this.searchDTO.department = null;
    this.worker = null;
    this.getDeparts();
    this.getAllWorker();
  }

  // 获取配送人员
  getAllWorker(e?, those?) {
    let that = those || this;
    let postData = {
      user: {
        name: e || "",
        hospital: { id: that.hospital },
        usertype: { id: 106 }, //配送人员
      },
      idx: 0,
      sum: 20,
    };
    that.isLoading = true;
    that.mainService
      .getFetchDataList("data", "user", postData)
      .subscribe((data) => {
        that.allWorker = data.list;
        that.isLoading = false;
      });
  }

  onCalendarChangeDate(dateArr){
    console.log(dateArr)
    if(dateArr.length == 2){
      let dateStart = new Date(dateArr[0]);
      let dateEnd = new Date(dateArr[1]);
      dateStart.setHours(0,0,0);
      dateEnd.setHours(23,59,59);
      this.searchDTO.dateRange = [dateStart,dateEnd];
    }
  }
  // 表格数据
  loading1 = false;
  getList() {
    var that = this;
    let data: any = {
      idx: that.pageIndex - 1,
      sum: that.pageSize,
      workOrder: {
        worker: { id: that.worker },
        gdState: { id: that.gdState },
        gdcode: this.searchDTO.gdcode,
        createDept: that.searchDTO.department,
        hosId: that.hospital,
      },
    };

    if (!data.workOrder.worker.id) {
      delete data.workOrder.worker;
    }
    if (!data.workOrder.gdState.id) {
      delete data.workOrder.gdState;
    }
    if (
      data.workOrder.specialCloseFlag === undefined ||
      data.workOrder.specialCloseFlag === null
    ) {
      delete data.workOrder.specialCloseFlag;
    }
    if (!data.workOrder.createDept) {
      delete data.workOrder.createDept;
    }
    delete data.workOrder.overdueTime24;
    delete data.workOrder.startTime1;
    delete data.workOrder.endTime1;
    if (false) {
      data.workOrder.overdueTime24 = true;
    } else if (that.startDate && that.endDate) {
      data.workOrder.startTime1 = that.startDate;
      data.workOrder.endTime1 = that.endDate;
    }
    that.loading1 = true;
    that.mainService
      .getFetchDataList("data", "workOrder", data)
      .subscribe((result) => {
        that.loading1 = false;
        that.listOfData = result.list.map(v => ({...v, endDeptsName: v.endDepts ? v.endDepts.map(v => v.dept).toString() : ''}));
        that.listLength = result.totalNum;
      });
  }

  // 获取所有科室
  getDeparts(dept?) {
    let data = {
      department: {
        cascadeHosId: this.hospital,
        dept: dept,
      },
      idx: 0,
      sum: 20,
    };
    this.isLoading = true;
    this.mainService
      .getFetchDataList("data", "department", data)
      .subscribe((data) => {
        this.alldepart = data.list;
        this.isLoading = false;
        if (dept === undefined && this.coopBtns.currentDept && !this.coopBtns.allOrders) {
          // 初次渲染，权限有当前科室工单,没有全部工单，则回显申请科室
          this.alldepart = [this.loginUser.dept];
          this.searchDTO.department = this.loginUser.dept.id;
        }
        if (dept === undefined && this.coopBtns.currentUser && !this.coopBtns.allOrders) {
          // 初次渲染，权限有当前人员工单,没有全部工单，则回显执行配送人员
          this.allWorker = [this.loginUser];
          this.worker = this.loginUser.id;
        }
        this.getList();
      });
  }

  // 获取工单状态
  getGdStates() {
    var that = this;
    that.mainService.getDictionary("list", "gdstate").subscribe((data) => {
      that.gdStates = data;
    });
  }

  // 日期选择
  startDate: string; //发起时间开始
  endDate: string; //发起时间结束
  changeDate(result?): void {
    if (!result) {
      this.startDate = this.endDate = "";
      return;
    }
    this.startDate = format(result[0], 'yyyy-MM-dd HH:mm:ss');
    this.endDate = format(result[1], 'yyyy-MM-dd HH:mm:ss');
  }

  // 展示信息提示框（con:提示信息，success:操作是否成功，promptInfo:操作结果提示信息）
  showPromptModal(con, success, promptInfo?) {
    this.promptModalShow = false;
    this.promptContent = con;
    this.ifSuccess = success;
    this.promptInfo = promptInfo;
    setTimeout(() => {
      this.promptModalShow = true;
    }, 100);
    this.getList();
  }

  // 查看
  detail(e, id) {
    e.stopPropagation();
    this.router.navigateByUrl("/main/orderManagement/orderDetail/" + id);
  }

  //删除
  loading3 = false;
  delModal: boolean = false; //删除模态框
  showDelModal(e, id) {
    e.stopPropagation();
    this.delModal = true;
    this.coopId = id + "";
  }
  hideDelModal() {
    this.delModal = false;
  }
  confirmDel() {
    this.loading3 = true;
    this.mainService[this.coopId.includes(",") ? "delOrders" : "delOrder"](
      this.coopId
    ).subscribe((data) => {
      this.loading3 = false;
      this.delModal = false;
      if (data.status == 200) {
        if (
          this.listOfData.length == 1 &&
          this.pageIndex == Math.ceil(this.listLength / this.pageSize)
        ) {
          this.listLength--;
          this.pageIndex = Math.ceil(this.listLength / this.pageSize);
        }
        this.showPromptModal("删除", true, "");
      } else {
        this.showPromptModal("删除", false, data.msg);
      }
    });
  }

  // 导出
  loading2 = false;
  export() {
    // 有备注，无发起时间则提示
    if(!this.searchDTO.dateRange.length){
      this.message.info('请选择发起时间！');
      return;
    }
    // 三个权限都没有，则不展示
    if(!this.coopBtns.currentDept && !this.coopBtns.currentUser && !this.coopBtns.allOrders){
      return;
    }
    let that = this;
    let postData: any = {
      idx: that.pageIndex - 1,
      sum: that.pageSize,
      workOrder: {
        worker: { id: that.worker },
        gdState: { id: that.gdState },
        gdcode: this.searchDTO.gdcode,
        createDept: that.searchDTO.department,
        hosId: that.hospital,
      },
    };

    if (!postData.workOrder.worker.id) {
      delete postData.workOrder.worker;
    }
    if (!postData.workOrder.gdState.id) {
      delete postData.workOrder.gdState;
    }
    if (
      postData.workOrder.specialCloseFlag === undefined ||
      postData.workOrder.specialCloseFlag === null
    ) {
      delete postData.workOrder.specialCloseFlag;
    }
    if (!postData.workOrder.createDept) {
      delete postData.workOrder.createDept;
    }
    delete postData.workOrder.overdueTime24;
    delete postData.workOrder.startTime1;
    delete postData.workOrder.endTime1;
    if (false) {
      postData.workOrder.overdueTime24 = true;
    } else if (that.startDate && that.endDate) {
      postData.workOrder.startTime1 = that.startDate;
      postData.workOrder.endTime1 = that.endDate;
    }
    this.loading2 = true;
    that.mainService.dataExport("workOrder", postData).subscribe(
      (data) => {
        this.loading2 = false;
        this.showPromptModal("导出", true, "");
        var file = new Blob([data], {
          type: "application/vnd.ms-excel",
        });
        //trick to download store a file having its URL
        var fileURL = URL.createObjectURL(file);
        var a = document.createElement("a");
        a.href = fileURL;
        a.target = "_blank";
        a.download = "工单列表.xls";
        document.body.appendChild(a);
        a.click();
      },
      (err) => {
        this.loading2 = false;
        this.showPromptModal("导出", false, "");
      }
    );
  }

  // 申请科室边输边搜节流阀
  isLoading = false;
  changeInp(e) {
    this.searchTimer(this.getDeparts, e);
  }

  // 用户输入搜索配送人员
  changeUser(e) {
    this.searchTimer(this.getAllWorker, e);
  }

  // 边输入边搜索节流阀
  searchTimer(fun, e) {
    this.isLoading = true;
    this.searchTimerSubject.next([fun, e]);
  }

  // 截取意见内容(ie内核截取)
  spliceContent(con) {
    if (con.length >= 41 && navigator.userAgent.indexOf("Trident") > -1) {
      return con.slice(0, 20) + "...";
    } else {
      return con;
    }
  }

  // 详细搜索
  searchModal: boolean = false; //新增/编辑模态框
  modelName = ""; //模态框名称
  add: boolean; //true:新增；false:编辑
  validateSearchForm: FormGroup; //新增/编辑表单
  coopData: any; //当前操作列
  // 新增弹框
  addModal() {
    this.modelName = "新增";
    this.add = true; //新增
    this.searchModal = true;
    this.initSearchForm();
  }
  //关闭新增/编辑弹框
  hideAddModal() {
    this.searchModal = false;
    this.initSearchForm();
  }

  // 初始化新增form表单
  initSearchForm() {
    this.validateSearchForm = this.fb.group({
      subType: [null],
      description: [''],
      supplierIds: [[]],
      manufacturerIds: [[]],
    });
  }
}
